
FUNCTION GKVs1D::PDF, nStd = nn, xmin=xl, xmax = xu, nbins = nb
;
; Purpose:
;
;	Creates a probability distribution function object (GKVpdf) from
;	from the data of 'self'.  The resulting GKVpdf object describes the
;	pdf of the bare values in 'self' (there is no effort to group events,
;	etc.  If this is desired, see GKVs2D::GKVpdf)
;
; Input Keywords:
;
;	nStd	Determine range of output independent variable by specifying the 
;		number of standart deviations (of 'self.values') desired.
;		Defaults to 8. (Optional)
;
;	xmin,	Determine range of output independent varialbe by specifying 
;	xmax	'xmin' and 'xmax'.  Default is to center range at mean of
;		'self.values', with a range of 4*sigma. (Optional).
;
;	nBins	Specify number of bins to use when calling "HIISTOGRAM" to
;		form the PDF.  Default is to use SQRT(nPoints), 
;		where 'nPoiints' is the number of points is the number of 
;		elements in 'self.values'. (Optional)
;
;
; Written by W.M. Nevins
;	6/16/01
;
valuePtr = self -> GetValues(/Open)
values = *valuePtr
PTR_FREE, valuePtr
IF(Query_COMPLEX(values)) THEN values=FLOAT(values)
nPoints = N_ELEMENTS(values)
moments = MOMENT(values)
std = SQRT(moments[1])
nBins = FIX(SQRT(nPoints))
IF QUERY_INTEGER(nb) THEN nBins = nb
PRINT, 'number of bins for PDF = ', nBins
nStd = 4.
IF(N_ELEMENTS(nn) EQ 1) THEN nStd = nn/2. > 1.
xmin = moments[0] - nStd*std
IF (N_ELEMENTS(xl) EQ 1) THEN xmin = xl
xmax = moments[0] + nStd*std
IF(N_ELEMENTS(xu) EQ 1) THEN xmax = xu
binSize = (xmax - xmin)/(nBins -1.)
binCenters = xmin + binSize/2 + binSize*FINDGEN(nBINS)
pdf = HISTOGRAM(values, NBINS=nbins, MAX=xmax, min=XMIN)
pdf = FLOAT(pdf)
errorBars = (pdf/SQRT(pdf-1))/(nPoints*binSize)
pdf = pdf/(nPoints*binSize)

out = {GKVs1D}
FOR i=0, N_TAGS({GKVsd}) DO out.(i) = self.(i)
out.title = 'PDF(' + self.title + ')'
out.mnemonic = self.mnemonic + '_pdf'
out.indices = PTR_NEW(['*'])
out.values = PTR_NEW(pdf)
vmax = MAX(pdf)
out.vrange = [0., vmax]
out.ErrorBars = PTR_NEW(errorBars)
grid1 = {Grid}
grid1.values = PTR_NEW(binCenters)
grid1.range = [xmin, xmax]
grid1.irange = [0,nBins-1]
grid1.units = self.units
grid1.title = self.title
grid1.mnemonic = self.mnemonic
out.grid1 = grid1
output = OBJ_NEW('GKVs1D', out)
RETURN, output
END ; ****** GKVs1D::GKVpdf ****** ;
